/**********************************************************
*
*	FILE				: main.c
*
*	AUTHOR			: Axiom Manufacturing
*						  Feb 2009    					     
*
*	PURPOSE			: Motor Control Demonstration for HY64 based on 
*						  MC10B8CV1 Motor Controller Block Description
*						  
*	This demo is specifically design for VID23-05 stepper motor.
*  Using a different motor may require code modification.
*
*	This demo moves motor to zero positon then reads POT input 
*	to get next positon.  If no change then pointer is not moved.  
*
*	Pointer movement direction is determined by comparing current 
*	pointer position with new pointer position.  
* 
**********************************************************/
#include <hidef.h>					// common defines and macros
#include <mc9s12hy64.h>				// derivative information
#include "sci_hc12.h"				// serial COM functions
#include "cpu.h"						// MCU configuration functions
#include "motor_drv.h"				// Motor Controller functions


void main(void) {

	DisableInterrupts;
	CPMU_PEE_4MHz();					// init clock module 4MHz bus clock
	init_atd_ch0();					// init ATD, CH0 for POT
	Init_Timer1();						// init timer1
	Configure_Ports();	
	MC_init();							// init motor controller for VID23-05 motor
	EnableInterrupts;
	MC_zero_pointer();				// motor pointer to zero position
	cur_ptr_pos = 0;					// init pointer position var 
	
	for(;;) {
		new_ptr_pos = (read_atd_ch0() / 3);	// get current POT positon	
														// result div 3 uses complete POT range

		if(new_ptr_pos > cur_ptr_pos){
			MC_step_dir(CW);			// set ptr movement directon
			steps_to_move = (new_ptr_pos - cur_ptr_pos);	// get # steps
		} else {
			MC_step_dir(CCW);			// set ptr movement directon
			steps_to_move = (cur_ptr_pos - new_ptr_pos);	// get # steps 
		}
		
		if(steps_to_move > 5)		// eliminate ptr jitter	
			MC_step(steps_to_move);	// move pointer
		cur_ptr_pos = new_ptr_pos;	// save ptr position
		
		Delay(25);						// delay	
	}
}
